@@ -29,7 +29,7 @@ module DriverPipeline (
 import Packages
 import HeaderInfo
 import DriverPhases
-import SysTools		( newTempName, addFilesToClean, copy )
+import SysTools
 import qualified SysTools	
 import HscMain
 import Finder
@@ -74,7 +74,8 @@ import Maybe
 preprocess :: DynFlags -> (FilePath, Maybe Phase) -> IO (DynFlags, FilePath)
 preprocess dflags (filename, mb_phase) =
   ASSERT2(isJust mb_phase || isHaskellSrcFilename filename, text filename) 
-  runPipeline anyHsc dflags (filename, mb_phase) Temporary Nothing{-no ModLocation-}
+  runPipeline anyHsc dflags (filename, mb_phase) 
+        Nothing Temporary Nothing{-no ModLocation-}
 
 -- ---------------------------------------------------------------------------
 -- Compile
@@ -168,7 +169,9 @@ compile hsc_env mod_summary maybe_old_linkable old_iface mod_index nmods = do
                             -> return ([], ms_hs_date mod_summary)
                           -- We're in --make mode: finish the compilation pipeline.
                           _other
-                            -> do runPipeline StopLn dflags (output_fn,Nothing) Persistent
+                            -> do runPipeline StopLn dflags (output_fn,Nothing)
+                                              (Just basename)
+                                              Persistent
                                               (Just location)
                                   -- The object filename comes from the ModLocation
                                   o_time <- getModificationTime object_filename
@@ -237,7 +240,7 @@ compileStub dflags mod location = do
 
 	-- compile the _stub.c file w/ gcc
 	let (stub_c,_) = mkStubPaths dflags (moduleName mod) location
-	runPipeline StopLn dflags (stub_c,Nothing) 
+	runPipeline StopLn dflags (stub_c,Nothing)  Nothing
 		(SpecificFile stub_o) Nothing{-no ModLocation-}
 
 	return stub_o
@@ -360,7 +363,8 @@ compileFile dflags stop_phase (src, mb_phase) = do
 			other      -> stop_phase
 
    (_, out_file) <- runPipeline stop_phase' dflags
-			  (src, mb_phase) output Nothing{-no ModLocation-}
+			  (src, mb_phase) Nothing output 
+                          Nothing{-no ModLocation-}
    return out_file
 
 
@@ -406,13 +410,16 @@ runPipeline
   :: Phase		        -- When to stop
   -> DynFlags		        -- Dynamic flags
   -> (FilePath,Maybe Phase)     -- Input filename (and maybe -x suffix)
+  -> Maybe FilePath             -- original basename (if different from ^^^)
   -> PipelineOutput	        -- Output filename
   -> Maybe ModLocation          -- A ModLocation, if this is a Haskell module
   -> IO (DynFlags, FilePath)	-- (final flags, output filename)
 
-runPipeline stop_phase dflags (input_fn, mb_phase) output maybe_loc
+runPipeline stop_phase dflags (input_fn, mb_phase) mb_basename output maybe_loc
   = do
-  let (basename, suffix) = splitFilename input_fn
+  let (input_basename, suffix) = splitFilename input_fn
+      basename | Just b <- mb_basename = b
+               | otherwise             = input_basename
 
 	-- If we were given a -x flag, then use that phase to start from
       start_phase
@@ -443,17 +450,19 @@ runPipeline stop_phase dflags (input_fn, mb_phase) output maybe_loc
   -- Sometimes, a compilation phase doesn't actually generate any output
   -- (eg. the CPP phase when -fcpp is not turned on).  If we end on this
   -- stage, but we wanted to keep the output, then we have to explicitly
-  -- copy the file.
+  -- copy the file, remembering to prepend a {-# LINE #-} pragma so that
+  -- further compilation stages can tell what the original filename was.
   case output of
     Temporary -> 
 	return (dflags', output_fn)
     _other ->
 	do final_fn <- get_output_fn dflags' stop_phase maybe_loc
-	   when (final_fn /= output_fn) $
-		  copy dflags ("Copying `" ++ output_fn ++ "' to `" ++ final_fn
-			++ "'") output_fn final_fn
+	   when (final_fn /= output_fn) $ do
+              let msg = ("Copying `" ++ output_fn ++"' to `" ++ final_fn ++ "'")
+                  line_prag = Just ("{-# LINE 1 \"" ++ input_fn ++ "\" #-}\n")
+	      copyWithHeader dflags msg line_prag output_fn final_fn
 	   return (dflags', final_fn)
-	        
+
 
 
 pipeLoop :: DynFlags -> Phase -> Phase 
@@ -789,7 +798,7 @@ runPhase Cmm stop dflags basename suff input_fn get_output_fn maybe_loc
 -- way too many hacks, and I can't say I've ever used it anyway.
 
 runPhase cc_phase stop dflags basename suff input_fn get_output_fn maybe_loc
-   | cc_phase `eqPhase` Cc || cc_phase `eqPhase` HCc
+   | cc_phase `eqPhase` Cc || cc_phase `eqPhase` Ccpp || cc_phase `eqPhase` HCc
    = do	let cc_opts = getOpts dflags opt_c
 	    hcc = cc_phase `eqPhase` HCc
 
@@ -851,7 +860,8 @@ runPhase cc_phase stop dflags basename suff input_fn get_output_fn maybe_loc
 		-- compiling .hc files, by adding the -x c option.
 		-- Also useful for plain .c files, just in case GHC saw a 
 		-- -x c option.
-			[ SysTools.Option "-x", SysTools.Option "c"] ++
+			[ SysTools.Option "-x", if cc_phase `eqPhase` Ccpp
+                                                then SysTools.Option "c++" else SysTools.Option "c"] ++
 			[ SysTools.FileOption "" input_fn
 			, SysTools.Option "-o"
 			, SysTools.FileOption "" output_fn
@@ -880,6 +890,10 @@ runPhase cc_phase stop dflags basename suff input_fn get_output_fn maybe_loc
 		       ++ split_opt
 		       ++ include_paths
 		       ++ pkg_extra_cc_opts
+#ifdef HAVE_GCC_HAS_WRAPV
+                  -- We need consistent integer overflow (trac #952)
+               ++ ["-fwrapv"]
+#endif
 		       ))
 
 	return (next_phase, dflags, maybe_loc, output_fn)
