@@ -102,7 +102,15 @@ parseStaticFlags args = do
 
   (more_leftover, errs) <- processArgs static_flags (unreg_flags ++ way_flags)
   when (not (null errs)) $ ghcError (UsageError (unlines errs))
-  return (more_leftover++leftover)
+
+     -- urgh.  -fexcess-precision is both a static flag and a dynamic
+     -- flag.  See opt_SimplExcessPrecision below.  if we let the static
+     -- flag parser grab it, then we'll never get the dynamic version,
+     -- so we have to duplicate it.
+  let even_more | "-fexcess-precision" `elem` args = ["-fexcess-precision"]
+		| otherwise                        = []
+
+  return (even_more++more_leftover++leftover)
 
 
 -- note that ordering is important in the following list: any flag which
@@ -263,7 +271,7 @@ opt_LiberateCaseThreshold	= lookup_def_int "-fliberate-case-threshold" (10::Int)
 opt_MaxWorkerArgs		= lookup_def_int "-fmax-worker-args" (10::Int)
 
 opt_GranMacros			= lookUp  FSLIT("-fgransim")
-opt_HiVersion			= read (cProjectVersionInt ++ cProjectPatchLevel) :: Int
+opt_HiVersion			= read (cProjectVersionInt ++ cProjectPatchLevel) :: Integer
 opt_HistorySize			= lookup_def_int "-fhistory-size" 20
 opt_OmitBlackHoling		= lookUp  FSLIT("-dno-black-holing")
 opt_RuntimeTypes		= lookUp  FSLIT("-fruntime-types")
