@@ -1,4 +1,4 @@
-{-# OPTIONS -fno-implicit-prelude -#include "HsBase.h" #-}
+{-# OPTIONS_GHC -fno-implicit-prelude -#include "HsBase.h" #-}
 
 #undef DEBUG_DUMP
 
@@ -16,6 +16,7 @@
 --
 -----------------------------------------------------------------------------
 
+-- #hide
 module GHC.IO ( 
    hWaitForInput, hGetChar, hGetLine, hGetContents, hPutChar, hPutStr,
    commitBuffer',	-- hack, see below
@@ -27,8 +28,6 @@ module GHC.IO (
    memcpy_baoff_ptr,
  ) where
 
-#include "config.h"
-
 import Foreign
 import Foreign.C
 
@@ -47,7 +46,7 @@ import GHC.Show
 import GHC.List
 import GHC.Exception    ( ioError, catch )
 
-#ifdef mingw32_TARGET_OS
+#ifdef mingw32_HOST_OS
 import GHC.Conc
 #endif
 
@@ -65,13 +64,15 @@ import GHC.Conc
 -- or 'False' if no input is available within @t@ milliseconds.
 --
 -- If @t@ is less than zero, then @hWaitForInput@ waits indefinitely.
--- NOTE: in the current implementation, this is the only case that works
--- correctly (if @t@ is non-zero, then all other concurrent threads are
--- blocked until data is available).
 --
 -- This operation may fail with:
 --
 --  * 'isEOFError' if the end of file has been reached.
+--
+-- NOTE for GHC users: unless you use the @-threaded@ flag,
+-- @hWaitForInput t@ where @t >= 0@ will block all other Haskell
+-- threads for the duration of the call.  It behaves like a
+-- @safe@ foreign call in this respect.
 
 hWaitForInput :: Handle -> Int -> IO Bool
 hWaitForInput h msecs = do
@@ -89,12 +90,13 @@ hWaitForInput h msecs = do
 	        writeIORef ref buf'
 		return True
 	else do r <- throwErrnoIfMinus1Retry "hWaitForInput" $
-	  		inputReady (fromIntegral (haFD handle_)) 
-			   (fromIntegral msecs) (haIsStream handle_)
+	  		inputReady (fromIntegral (haFD handle_))
+			   (fromIntegral msecs)
+                           (fromIntegral $ fromEnum $ haIsStream handle_)
 		return (r /= 0)
 
 foreign import ccall safe "inputReady"
-  inputReady :: CInt -> CInt -> Bool -> IO CInt
+  inputReady :: CInt -> CInt -> CInt -> IO CInt
 
 -- ---------------------------------------------------------------------------
 -- hGetChar
@@ -402,15 +404,15 @@ unpackAcc buf (I# r) (I# len) acc = IO $ \s -> unpack acc (len -# 1#) s
 --  * 'isPermissionError' if another system resource limit would be exceeded.
 
 hPutChar :: Handle -> Char -> IO ()
-hPutChar handle c = 
-    c `seq` do   -- must evaluate c before grabbing the handle lock
+hPutChar handle c = do
+    c `seq` return ()
     wantWritableHandle "hPutChar" handle $ \ handle_  -> do
     let fd = haFD handle_
     case haBufferMode handle_ of
 	LineBuffering    -> hPutcBuffered handle_ True  c
 	BlockBuffering _ -> hPutcBuffered handle_ False c
 	NoBuffering      ->
-		withObject (castCharToCChar c) $ \buf -> do
+		with (castCharToCChar c) $ \buf -> do
   		  writeRawBufferPtr "hPutChar" (fromIntegral fd) (haIsStream handle_) buf 0 1
 		  return ()
 
@@ -563,7 +565,7 @@ commitBuffer
 
 commitBuffer hdl raw sz@(I# _) count@(I# _) flush release = do
   wantWritableHandle "commitAndReleaseBuffer" hdl $
-     commitBuffer' hdl raw sz count flush release
+     commitBuffer' raw sz count flush release
 
 -- Explicitly lambda-lift this function to subvert GHC's full laziness
 -- optimisations, which otherwise tends to float out subexpressions
@@ -576,7 +578,7 @@ commitBuffer hdl raw sz@(I# _) count@(I# _) flush release = do
 --
 -- This hack is a fairly big win for hPutStr performance.  --SDM 18/9/2001
 --
-commitBuffer' hdl raw sz@(I# _) count@(I# _) flush release
+commitBuffer' raw sz@(I# _) count@(I# _) flush release
   handle_@Handle__{ haFD=fd, haBuffer=ref, haBuffers=spare_buf_ref } = do
 
 #ifdef DEBUG_DUMP
@@ -599,7 +601,7 @@ commitBuffer' hdl raw sz@(I# _) count@(I# _) flush release
 
 		-- not flushing, and there's enough room in the buffer:
 		-- just copy the data in and update bufWPtr.
-	    then do memcpy_baoff_ba old_raw w raw (fromIntegral count)
+	    then do memcpy_baoff_ba old_raw (fromIntegral w) raw (fromIntegral count)
 		    writeIORef ref old_buf{ bufWPtr = w + count }
 		    return (newEmptyBuffer raw WriteBuffer sz)
 
@@ -693,7 +695,7 @@ bufWrite fd ref is_stream ptr count can_block =
   if (size - w > count)
 	-- There's enough room in the buffer:
 	-- just copy the data in and update bufWPtr.
-	then do memcpy_baoff_ptr old_raw w ptr (fromIntegral count)
+	then do memcpy_baoff_ptr old_raw (fromIntegral w) ptr (fromIntegral count)
 	        writeIORef ref old_buf{ bufWPtr = w + count }
 		return count
 
@@ -727,7 +729,7 @@ writeChunkNonBlocking fd is_stream ptr bytes = loop 0 bytes
   loop :: Int -> Int -> IO Int
   loop off bytes | bytes <= 0 = return off
   loop off bytes = do
-#ifndef mingw32_TARGET_OS
+#ifndef mingw32_HOST_OS
     ssize <- c_write (fromIntegral fd) (ptr `plusPtr` off) (fromIntegral bytes)
     let r = fromIntegral ssize :: Int
     if (r == -1)
@@ -790,18 +792,18 @@ bufRead fd ref is_stream ptr so_far count =
   	let avail = w - r
 	if (count == avail)
 	   then do 
-		memcpy_ptr_baoff ptr raw r (fromIntegral count)
+		memcpy_ptr_baoff ptr raw (fromIntegral r) (fromIntegral count)
 		writeIORef ref buf{ bufWPtr=0, bufRPtr=0 }
 		return (so_far + count)
 	   else do
 	if (count < avail)
 	   then do 
-		memcpy_ptr_baoff ptr raw r (fromIntegral count)
+		memcpy_ptr_baoff ptr raw (fromIntegral r) (fromIntegral count)
 		writeIORef ref buf{ bufRPtr = r + count }
 		return (so_far + count)
 	   else do
   
-	memcpy_ptr_baoff ptr raw r (fromIntegral avail)
+	memcpy_ptr_baoff ptr raw (fromIntegral r) (fromIntegral avail)
 	writeIORef ref buf{ bufWPtr=0, bufRPtr=0 }
 	let remaining = count - avail
 	    so_far' = so_far + avail
@@ -873,18 +875,18 @@ bufReadNonBlocking fd ref is_stream ptr so_far count =
   	let avail = w - r
 	if (count == avail)
 	   then do 
-		memcpy_ptr_baoff ptr raw r (fromIntegral count)
+		memcpy_ptr_baoff ptr raw (fromIntegral r) (fromIntegral count)
 		writeIORef ref buf{ bufWPtr=0, bufRPtr=0 }
 		return (so_far + count)
 	   else do
 	if (count < avail)
 	   then do 
-		memcpy_ptr_baoff ptr raw r (fromIntegral count)
+		memcpy_ptr_baoff ptr raw (fromIntegral r) (fromIntegral count)
 		writeIORef ref buf{ bufRPtr = r + count }
 		return (so_far + count)
 	   else do
 
-	memcpy_ptr_baoff ptr raw r (fromIntegral avail)
+	memcpy_ptr_baoff ptr raw (fromIntegral r) (fromIntegral avail)
 	writeIORef ref buf{ bufWPtr=0, bufRPtr=0 }
 	let remaining = count - avail
 	    so_far' = so_far + avail
@@ -901,7 +903,7 @@ bufReadNonBlocking fd ref is_stream ptr so_far count =
 
 readChunkNonBlocking :: FD -> Bool -> Ptr a -> Int -> IO Int
 readChunkNonBlocking fd is_stream ptr bytes = do
-#ifndef mingw32_TARGET_OS
+#ifndef mingw32_HOST_OS
     ssize <- c_read (fromIntegral fd) (castPtr ptr) (fromIntegral bytes)
     let r = fromIntegral ssize :: Int
     if (r == -1)
@@ -937,13 +939,13 @@ slurpFile fname = do
 -- memcpy wrappers
 
 foreign import ccall unsafe "__hscore_memcpy_src_off"
-   memcpy_ba_baoff :: RawBuffer -> RawBuffer -> Int -> CSize -> IO (Ptr ())
+   memcpy_ba_baoff :: RawBuffer -> RawBuffer -> CInt -> CSize -> IO (Ptr ())
 foreign import ccall unsafe "__hscore_memcpy_src_off"
-   memcpy_ptr_baoff :: Ptr a -> RawBuffer -> Int -> CSize -> IO (Ptr ())
+   memcpy_ptr_baoff :: Ptr a -> RawBuffer -> CInt -> CSize -> IO (Ptr ())
 foreign import ccall unsafe "__hscore_memcpy_dst_off"
-   memcpy_baoff_ba :: RawBuffer -> Int -> RawBuffer -> CSize -> IO (Ptr ())
+   memcpy_baoff_ba :: RawBuffer -> CInt -> RawBuffer -> CSize -> IO (Ptr ())
 foreign import ccall unsafe "__hscore_memcpy_dst_off"
-   memcpy_baoff_ptr :: RawBuffer -> Int -> Ptr a -> CSize -> IO (Ptr ())
+   memcpy_baoff_ptr :: RawBuffer -> CInt -> Ptr a -> CSize -> IO (Ptr ())
 
 -----------------------------------------------------------------------------
 -- Internal Utils
