@@ -23,6 +23,7 @@ module Language.Haskell.TH.Syntax(
 
 	-- Names
 	Name(..), mkName, newName, nameBase, nameModule,
+    showName, showName', NameIs(..),
 
 	-- The algebraic data types
 	Dec(..), Exp(..), Con(..), Type(..), Cxt, Match(..), 
@@ -50,6 +51,7 @@ import Data.IORef
 import GHC.IOBase	( unsafePerformIO )
 import Control.Monad (liftM)
 import System.IO	( hPutStrLn, stderr )
+import Data.Char        ( isAlpha )
 
 -----------------------------------------------------
 --
@@ -428,21 +430,48 @@ instance Ord NameFlavour where
 					    (m1 `compare` m2) 
   (NameG _ _ _)    `compare` other	  = GT
 
-instance Show Name where
-  show (Name occ NameS) = occString occ
+data NameIs = Alone | Applied | Infix
 
+showName :: Name -> String
+showName = showName' Alone
+
+showName' :: NameIs -> Name -> String
+showName' ni nm
+ = case ni of
+       Alone        -> nms
+       Applied
+        | pnam      -> nms
+        | otherwise -> "(" ++ nms ++ ")"
+       Infix
+        | pnam      -> "`" ++ nms ++ "`"
+        | otherwise -> nms
+    where
 	-- For now, we make the NameQ and NameG print the same, even though
 	-- NameQ is a qualified name (so what it means depends on what the
 	-- current scope is), and NameG is an original name (so its meaning
 	-- should be independent of what's in scope.
 	-- We may well want to distinguish them in the end.
-  show (Name occ (NameQ m))      = modString m ++ "." ++ occString occ
-  show (Name occ (NameG ns p m)) = modString m ++ "." ++ occString occ
+	-- Ditto NameU and NameL
+        nms = case nm of
+                    Name occ NameS          -> occString occ
+                    Name occ (NameQ m)      -> modString m ++ "." ++ occString occ
+                    Name occ (NameG ns p m) -> modString m ++ "." ++ occString occ
+                    Name occ (NameU u)      -> occString occ ++ "_" ++ show (I# u)
+                    Name occ (NameL u)      -> occString occ ++ "_" ++ show (I# u)
 
-	-- Ditto NameU and NameL  
-  show (Name occ (NameU u)) = occString occ ++ "_" ++ show (I# u)
-  show (Name occ (NameL u)) = occString occ ++ "_" ++ show (I# u)
+        pnam = classify nms
 
+        -- True if we are function style, e.g. f, [], (,)
+        -- False if we are operator style, e.g. +, :+
+        classify "" = False -- shouldn't happen; . operator is handled below
+        classify (x:xs) | isAlpha x || (x `elem` "_[]()") =
+                            case dropWhile (/='.') xs of
+                                  (_:xs') -> classify xs'
+                                  []      -> True
+                        | otherwise = False
+
+instance Show Name where
+  show = showName
 
 -- 	Tuple data and type constructors
 tupleDataName  :: Int -> Name	-- Data constructor
