@@ -749,8 +749,8 @@ static StgBool check_read_only(StgTRecHeader *trec STG_UNUSED) {
       s = e -> tvar;
       if (entry_is_read_only(e)) {
         TRACE("%p : check_read_only for TVar %p, saw %ld", trec, s, e -> num_updates);
-        if (s -> num_updates != e -> num_updates) {
-          // ||s -> current_value != e -> expected_value) {
+        if (s -> num_updates != e -> num_updates
+          ||s -> current_value != e -> expected_value) {
           TRACE("%p : mismatch\n", trec);
           result = FALSE;
           BREAK_FOR_EACH;
@@ -1013,7 +1013,6 @@ StgBool stmCommitNestedTransaction(Capability *cap, StgTRecHeader *trec) {
       // at the end of the call to validate_and_acquire_ownership.  This forms the
       // linearization point of the commit.
 
-      if (result) {
         TRACE("%p : read-check succeeded\n", trec);
         FOR_EACH_ENTRY(trec, e, {
           // Merge each entry into the enclosing transaction record, release all
@@ -1027,9 +1026,8 @@ StgBool stmCommitNestedTransaction(Capability *cap, StgTRecHeader *trec) {
           merge_update_into(cap, et, s, e -> expected_value, e -> new_value);
           ACQ_ASSERT(s -> current_value != (StgClosure *)trec);
         });
-      } else {
+    } else {
         revert_ownership(trec, FALSE);
-      }
     }
   } 
 
