@@ -282,6 +282,7 @@ freeStorage (void)
 #if defined(THREADED_RTS)
     closeMutex(&sm_mutex);
     closeMutex(&atomic_modify_mutvar_mutex);
+    stgFree(nurseries);
 #endif
 }
 
@@ -768,6 +769,7 @@ allocatePinned( nat n )
     if (bd == NULL || (bd->free + n) > (bd->start + BLOCK_SIZE_W)) {
 	pinned_object_block = bd = allocBlock();
 	dbl_link_onto(bd, &g0s0->large_objects);
+	g0s0->n_large_blocks++;
 	bd->gen_no = 0;
 	bd->step   = g0s0;
 	bd->flags  = BF_PINNED | BF_LARGE;
@@ -992,6 +994,11 @@ calcNeeded(void)
    in the page, and when the page is emptied (all objects on the page
    are free) we free the page again, not forgetting to make it
    non-executable.
+
+   TODO: The inability to handle objects bigger than BLOCK_SIZE_W means that
+         the linker cannot use allocateExec for loading object code files
+         on Windows. Once allocateExec can handle larger objects, the linker
+         should be modified to use allocateExec instead of VirtualAlloc.
    ------------------------------------------------------------------------- */
 
 static bdescr *exec_block;
@@ -1053,20 +1060,17 @@ void freeExec (void *addr)
     bd->gen_no -= *(StgPtr)p;
     *(StgPtr)p = 0;
 
-    // Free the block if it is empty, but not if it is the block at
-    // the head of the queue.
-    if (bd->gen_no == 0 && bd != exec_block) {
-	debugTrace(DEBUG_gc, "free exec block %p", bd->start);
-	if (bd->u.back) {
-	    bd->u.back->link = bd->link;
-	} else {
-	    exec_block = bd->link;
-	}
-	if (bd->link) {
-	    bd->link->u.back = bd->u.back;
-	}
-	setExecutable(bd->start, bd->blocks * BLOCK_SIZE, rtsFalse);
-	freeGroup(bd);
+    if (bd->gen_no == 0) {
+        // Free the block if it is empty, but not if it is the block at
+        // the head of the queue.
+        if (bd != exec_block) {
+            debugTrace(DEBUG_gc, "free exec block %p", bd->start);
+            dbl_link_remove(bd, &exec_block);
+            setExecutable(bd->start, bd->blocks * BLOCK_SIZE, rtsFalse);
+            freeGroup(bd);
+        } else {
+            bd->free = bd->start;
+        }
     }
 
     RELEASE_SM_LOCK
